{==========================================================================}
{ Program            : MOSSIM                                              }
{ Author             : Robert Lacoste, Chaville, France                    }
{ Version            : 1.3                                                 }
{ Release date       : 27/07/1996                                          }
{==========================================================================}

{--------------------------------------------------------------------------}
{                              UNIT AJOUTS                                 }
{                                                                          }
{ Description : Management of objets creation (adding to current network)  }
{                                                                          }
{--------------------------------------------------------------------------}

 UNIT AJOUTS;

   INTERFACE                    { External declarations }
     USES GRAPH,
          DATA,
          GDS,
          PROCGEN,
          DESSINS;

     { interconnecting of a transistor }
     PROCEDURE METTRTABLE(SS:SENSTRANSI; X,Y:INTEGER; PTR:PTTRANSISTOR);

     { adding of a transistor to the network }
     PROCEDURE FAITTRANSISTOR(ORIENTATION:INTEGER);

     { adding of a pullup to the network }
     PROCEDURE FAITPULLUP;

     { adding of an input to the network }
     PROCEDURE FAITENTREE;

     { adding of a supply (vss or vcc) to the network }
     PROCEDURE FAITALIM(GENRE:INTEGER);

     { adding of an output to the network }
     PROCEDURE FAITSORTIE;

     { adding of a free text to the network}
     PROCEDURE FAITTXT;

   IMPLEMENTATION               { private declarations }

     var z:string;              { temporary }


 {--------------- subfunction : interconnecting of a transistor ----------}

 PROCEDURE METTRTABLE{(SS:SENSTRANSI; X,Y:INTEGER; PTR:PTTRANSISTOR)};
   begin
 		case ss of
 		  d: begin
               { right sided }
 		       table[x+1,y].nconnect:=ptr^.grille;
 		       table[x-1,y-1].nconnect:=ptr^.t1;
 		       table[x-1,y+1].nconnect:=ptr^.t2;
 		     end;
 		  g: begin
               { left sided }
 		       table[x-1,y].nconnect:=ptr^.grille;
 		       table[x+1,y-1].nconnect:=ptr^.t1;
 		       table[x+1,y+1].nconnect:=ptr^.t2;
 		     end;
 		  b: begin
               { down sided }
 		       table[x,y+1].nconnect:=ptr^.grille;
 		       table[x-1,y-1].nconnect:=ptr^.t1;
 		       table[x+1,y-1].nconnect:=ptr^.t2;
 		     end;
 		  h: begin
               { up sided }
 		       table[x,y-1].nconnect:=ptr^.grille;
 		       table[x-1,y+1].nconnect:=ptr^.t1;
 		       table[x+1,y+1].nconnect:=ptr^.t2;
 		     end;
 		end;
   end;

 {---------------------- Adding of a transistor ---------------------------}

 PROCEDURE FAITTRANSISTOR  {(ORIENTATION:INTEGER)};
   var x,y:integer;
       i,index,bp:integer;
       conn:ptconn;
       ptr:pttransistor;
       ss:senstransi;
       s:string;
       chan:typetransi;
   begin
     modifeffectuee; ss:=efface;

     {select orientation}
     case orientation of
       1: begin Z:='TRANSISTOR RIGHT'; ss:=d end;
       2: begin Z:='TRANSISTOR LEFT'; ss:=g end;
       3: begin Z:='TRANSISTOR UP'; ss:=h end;
       4: begin Z:='TRANSISTOR DOWN'; ss:=b end;
     end;
     TITRE(Z);

     Z:='Position ?'; info(z,WHITE);

     {get screen position}
     bp:=litsouris(x,y);
     if ((x>1) and (y>1) and (x<DIMMX) and (y<DIMMY) and (bp=1)) then

      {check if room available on the grid}
      if testesilibre(x,y,1,1,1,1) then

       {check if memory available for transitor and nodes}
 	   if ilyadelaplace(1,3) then
   	     begin

           {ask for channel type}
           littxt(s,'Channel (N/P)',1,['p','P','n','N']);
           if (s[1] in ['p','P']) then
             chan:=pchannel
           else
             chan:=nchannel;

           if length(s) <> 0 then
            begin

             {draw the transistor}
 	         dessintransi(x,y,ss,chan);
 	         nbtr:=nbtr+1;
 	         index:=0; i:=1;

             {get free space in the transistor table}
 	         repeat
 	           if transistors[i]=nil then index:=i;
 	           i:=succ(i);
 	         until ((index<>0) or (i>NBMTRANSIS));
 	         if index=0 then erreur(98) else
 	           begin

                 {create and initialise a new transistor record}
  		         new(ptr); transistors[index]:=ptr;
 		         ptr^.orientation:=ss;
 		         ptr^.trx:=x; ptr^.try:=y;
                 ptr^.channel:=chan;

                 {grid node and its connection list}
 		         ptr^.grille:=faitnoeud(normal);
 		         new(conn); conn^.num:=index;
 		         conn^.connsuiv:=nil; conn^.broche:=tgr;
 		         noeuds[ptr^.grille]^.debutconnections:=conn;

                 {t1 node and its connection list}
 		         ptr^.t1:=faitnoeud(normal);
 		         new(conn); conn^.num:=index;
 		         conn^.connsuiv:=nil; conn^.broche:=tt1;
 		         noeuds[ptr^.t1]^.debutconnections:=conn;

                 {t2 node and its connection list}
 		         ptr^.t2:=faitnoeud(normal);
 		         new(conn); conn^.num:=index;
 		         conn^.connsuiv:=nil; conn^.broche:=tt2;
 		         noeuds[ptr^.t2]^.debutconnections:=conn;

                 {declare the transistor's screen position used}
 		         mettable(x,y,1,1,1,1,eltrans,index);
 		         mettrtable(ss,x,y,ptr);
 	          end;
 	       end;
       end;
   end;

 {-------------------------- Adding of a pullup ---------------------------}

 PROCEDURE FAITPULLUP;
   var a,x,y,bp:integer;
   begin
     modifeffectuee;
     Z:='PULL-UP'; titre(Z);
     z:='Position ?'; info(z,WHITE);

     {get pullup position}
     bp:=litsouris(x,y);
     if ((x>1) and (y>4) and (x<DIMMX) and (y<DIMMY) and (bp=1)) then

       {check if room available on the grid}
       if testesilibre(x,y,1,1,4,1) then

         {check if room available for the node}
   	     if ilyadelaplace(0,1) then
  	       begin

             {draw the pullup}
 	         dessinpullup(x,y);

             {create the node}
 	         a:=faitnoeud(normal);
 	         noeuds[a]^.xpull:=x; noeuds[a]^.ypull:=y;

             {declare the pullup's screen position used}
 	         mettable(x,y,1,1,5,1,elpull,a);
 	         table[x,y].nconnect:=a;
           end;
   end;

 {---------------------------- adding of an input ------------------------}

 PROCEDURE FAITENTREE;
   var a,x,y,n,bp:integer;
   begin
     modifeffectuee;
     Z:='INPUT';titre(Z);
     Z:='Position ?'; info(z,WHITE);

     {get input position}
     bp:=litsouris(x,y);
     if ((x>3) and (y>1) and (x<DIMMX) and (y<DIMMY) and (bp=1)) then

       {check if room available on the grid}
       if testesilibre(x,y,3,0,1,1) then

         {check if room available for the node}
      	 if ilyadelaplace(0,1) then
 	       begin

             {get input number}
 	         repeat n:=litnum(1) until ((N>=0) AND (n<=NBMENTREES));
             if (n>0) then
               if assignentrees[n]<>0 then erreur(3) else
 	             begin

                   {draw the input}
 		           dessinentree(x,y,n);

                   {create the node}
 	               a:=faitnoeud(entree);
 	        	   noeuds[a]^.numentree:=n;
 		           noeuds[a]^.xentr:=x; noeuds[a]^.yentr:=y;

                   {declare the screen position used and assign the input}
 		           mettable(x,y,3,0,1,1,elentr,a);
 		           assignentrees[n]:=a;
 		           table[x,y].nconnect:=a;
 	            end;
           end;
   end;

 {----------------------- adding of a power node (vss/vcc) ----------------}

 PROCEDURE FAITALIM  {(GENRE:INTEGER)};
   var a,x,y,bp:integer;
   begin
     modifeffectuee;
     case genre of
       1: Z:='VCC';
       2: Z:='VSS';
     end;
     TITRE(Z);
     Z:='Position ?'; info(z,WHITE);

     {get the power position}
     bp:=litsouris(x,y);
     if ((x>1) and (y>3) and (x<DIMMX) and (y<DIMMY-1) and (bp=1)) then

       {check if room available on the grid}
       if (((genre=1) and testesilibre(x,y,1,1,3,0)) or
 	       ((genre=2) and testesilibre(x,y,2,2,0,2))) then

         {check if memory available for the node}
    	if ilyadelaplace(0,1) then
 	      begin
 	        case genre of
 	          1: begin
                   {draw and create a vcc node}
 		           dessinvcc(x,y);
 	               a:=faitnoeud(vcc);
 		           noeuds[a]^.xalim:=x; noeuds[a]^.yalim:=y;
 		           mettable(x,y,1,1,3,0,elvcc,a);
 		           table[x,y].nconnect:=a;
 	             end;
 	          2: begin
                   {draw and create a vss node}
 		           dessinvss(x,y);
 	               a:=faitnoeud(vss);
 		           noeuds[a]^.xalim:=x; noeuds[a]^.yalim:=y;
 		           mettable(x,y,2,2,0,2,elvss,a);
 		           table[x,y].nconnect:=a;
 		         end;
 	      end;
 	  end;
   end;

 {------------------------ adding of an output ----------------------------}

 PROCEDURE FAITSORTIE;
   var a,x,y,n,bp:integer;
   begin
     modifeffectuee;
     Z:='OUTPUT ';titre(Z);
     Z:='Position ?'; info(z,WHITE);

     {get output position}
     bp:=litsouris(x,y);
     if ((x>2) and (y>1) and (x<DIMMX-3) and (y<DIMMY) and (bp=1)) then

       {check if room available on the grid}
       if testesilibre(x,y,0,3,1,1) then

         {check if memory available for the node}
 	     if ilyadelaplace(0,1) then
 	       begin

             {get the output number}
 	         repeat n:=litnum(1) until ((N>=0) AND (n<=NBMSORTIES));
             if (n>0) then
 	          if assignsorties[n]<>0 then erreur(2) else
 	           begin

                 {draw the output}
 		         dessinsortie(x,y,n);

                 {create the node}
 	             a:=faitnoeud(normal);
 		         noeuds[a]^.numsortie:=n;
 		         noeuds[a]^.xsort:=x; noeuds[a]^.ysort:=y;

                 {declare the screen position used and assign the output}
 		         mettable(x,y,0,3,1,1,elsort,a);
 		         table[x,y].nconnect:=a;
 		         assignsorties[n]:=a;
 	           end;
 	      end;
   end;

 {--------------------------- adding of a free text -----------------------}

 PROCEDURE FAITTXT;
   var a,x,y,n,bp:integer;
       ss:string;
   begin
     modifeffectuee;
     Z:='FREE TEXT '; titre(Z);
     Z:='Position ?'; info(z,WHITE);

     {get output position}
     bp:=litsouris(x,y);
     if ((x>0) and (y>1) and (x<DIMMX-1) and (y<=DIMMY) and (bp=1)) then

       {check if room available on the grid}
       if testesilibre(x,y,0,0,0,0) then

         begin
           {check if memory available for the text}
 	       n:=0;
           for i:=1 to NBMTEXTS do
             if ((n=0) and (freetexts[i].x=0) and (freetexts[i].y=0)) then
               n:=i;

           if (n=0) then
             erreur (44)
           else
 	       begin

             {get the string}
 	         littxt(ss, 'Txt', LTEXT, ['A'..'Z','a'..'z','0'..'9',
                    '&','#','(','-','_','[',']',')','=','+','?','.',';',
                    ':','/','!','$','*','<','>',' ']);
             freetexts[n].txt:=copy(ss,1,LTEXT);

 	         if (length(freetexts[n].txt) <> 0) then
 	           begin

                 {draw the text}
 		         dessintexte(x,y,freetexts[n].txt);

                 {store the coordinates}
                 freetexts[n].x := x;
                 freetexts[n].y := y;

                 {declare the screen position used}
 		         mettable(x,y,0,0,0,0,eltxt,n);
 	           end;
 	      end;
        end;
   end;


   {--------------------------- unit initialisation -----------------------}

   BEGIN
   END.

{==========================================================================}
{                              END OF FILE                                 }
{==========================================================================}
