{==========================================================================}
{ Program            : MOSSIM                                              }
{ Author             : Robert Lacoste, Chaville, France                    }
{ Version            : 1.3                                                 }
{ Release date       : 27/07/1996                                          }
{==========================================================================}

{--------------------------------------------------------------------------}
{                              UNIT CONNECT                                }
{                                                                          }
{ Description : Connections making and deleting                            }
{                                                                          }
{--------------------------------------------------------------------------}

 unit connect;

   interface                    {public declarations}
     uses GRAPH,
          CRT,
          DATA,
          GDS,
          PROCGEN;

     {graphical erasing of a connection}
     PROCEDURE EFFACESEGMENTS(TP:TABLEPNTS;NP:INTEGER);

     {mouse definition of connection}
     PROCEDURE CONNECTE;


   IMPLEMENTATION               {local declarations}


   {---------------------- graphical erasing of a connection --------------}
   {tp: table of connection segments, np:number of segments}

   PROCEDURE EFFACESEGMENTS{(VAR TP:TABLEPNTS;NP:INTEGER)};
   var i,j,n:integer;
   begin
     {color:black}
     cr:=sacwk(fschema);cr:=ssplci(0);

     {for each segment of the segment table}
     if (np>1) then for n:=1 to np-1 do
       begin

         {erase the segment}
 	     cr:=sln(xgrille(tp[n].x),ygrille(tp[n].y),
 		         xgrille(tp[n+1].x),ygrille(tp[n+1].y));

         {and redraw the grid markers under the segment}
 	     if (tp[n].x<tp[n+1].x) then
 	       for i:=tp[n].x to tp[n+1].x do
 	         begin
 	           if (tp[n].y<tp[n+1].y) then
 		         for j:=tp[n].y to tp[n+1].y do
                   cr:=smk(xgrille(i),ygrille(j))
 	           else
 		         for j:=tp[n+1].y to tp[n].y do
                   cr:=smk(xgrille(i),ygrille(j));
 	         end
 	     else
           for i:=tp[n+1].x to tp[n].x do
 	         begin
 	           if (tp[n].y<tp[n+1].y) then
 		         for j:=tp[n].y to tp[n+1].y do
                   cr:=smk(xgrille(i),ygrille(j))
 	           else
 		         for j:=tp[n+1].y to tp[n].y do
                   cr:=smk(xgrille(i),ygrille(j));
 	         end;
       end;

     {redraw the grid marker at the beginning of each segment}
     if np<>0 then
       for i:=1 to np do
         cr:=smk(xgrille(tp[i].x),ygrille(tp[i].y));
   end;

   {------------------ replace node n2 by node n1 -------------------------}

   function remplace (n2,n1:zonenoeud):boolean;
   var pc:ptconn;
       c:boolean;
       i,j:integer;
   begin

     {for each connection of node n2...}
     c:=true;
     pc:=noeuds[n2]^.debutconnections;
     while (pc<>nil) do with pc^ do
       begin

         {...replace each occurence of n2 with n1}
 	     with transistors[num]^ do
 		   case broche of
 		     tgr: if (grille<>n2) then begin c:=false;erreur(95); end
 		          else grille:=n1;
 		     tt1: if (t1<>n2)     then begin c:=false;erreur(95); end
 		          else t1:=n1;
 		     tt2: if (t2<>n2)     then begin c:=false;erreur(95); end
 		          else t2:=n1;
 		end;
 	    pc:=connsuiv;
      end;

     {for each grid coordinate...}
     for i:=1 to DIMMX do
       for j:=1 to DIMMY do

         {replace each occurence of n2 with n1}
 	     with table[i,j] do
 	       begin
 	         if (nconnect=n2) then nconnect:=n1;
 	         if element in [elpull,elsort,elentr,elvcc,elvss] then
 	           if (numnoeud=n2) then numnoeud:=n1;
 	       end;

     {replace n2 with n1 for all inputs and outputs assignements}
     for i:=1 to NBMENTREES do
       if (assignentrees[i]=n2) then
 		 assignentrees[i]:=n1;
     for i:=1 to NBMSORTIES do
       if (assignsorties[i]=n2) then
         assignsorties[i]:=n2;

     {return true if ok}
     remplace:=c;
   end;


   {--------------------- fusion of two nodes -----------------------------}
   {combine the two nodes n1 and n2 in only one (n1), and add the npoints}
   {segments declared in tablepts arry to the new node segments list}

   FUNCTION FUSIONNE(N1,N2:ZONENOEUD; TABLEPTS:TABLEPNTS;
                      NPOINTS:INTEGER):BOOLEAN;
   var n:ptnoeud;
       index,indexprecc:ptconn;
       sindex,sindexprecc:ptseg;
       nd1,nd2:noeud;
       i:integer;
       s:ptseg;
       f:boolean;

   begin

     {make local copies of the two nodes}
     nd1:=noeuds[n1]^; nd2:=noeuds[n2]^;
     f:=false;

     {check nodes compatibility}
     if ((nd1.classe=nul) or (nd2.classe=nul)) then erreur(88)

     {at least 1 of the 2 is a normal node (no input or power)}
     else if ((nd1.classe<>normal) and (nd2.classe<>normal)) then erreur(31)

     {maximum 1 of the 2 is pullup}
     else if ((nd1.xpull<>0) and (nd2.xpull<>0)) then erreur(32)

     {maximum 1 of the 2 is an output}
     else if ((nd1.numsortie<>0) and (nd2.numsortie<>0)) then erreur(33)
     else begin

       {create a new node}
 	   f:=true;
       new(n);

       {new node class is the predominent between the two nodes}
 	   if (nd1.classe<>normal) then n^.classe:=nd1.classe
           				       else n^.classe:=nd2.classe;

       {if one of the two nodes is an output, copy the caracteristics}
 	   n^.numsortie:=nd1.numsortie+nd2.numsortie;
 	   if (nd1.numsortie<>0) then
         begin
 			n^.xsort:=nd1.xsort;
 			n^.ysort:=nd1.ysort;
         end
 	   else
         begin
 			n^.xsort:=nd2.xsort;
 			n^.ysort:=nd2.ysort;
         end;

       {if one of the two nodes is a pullup, copy the caracteristics}
 	   n^.xpull:=nd1.xpull+nd2.xpull;
 	   n^.ypull:=nd1.ypull+nd2.ypull;
 	   n^.etat:=nondef;

       {if one of the two nodes is an input, copy input description}
 	   if (nd1.classe=entree) then
         begin
 			n^.numentree:=nd1.numentree;
 			n^.xentr:=nd1.xentr;
 			n^.yentr:=nd1.yentr;
         end
       else if (nd2.classe=entree) then
         begin
 			n^.numentree:=nd2.numentree;
 			n^.xentr:=nd2.xentr;
 			n^.yentr:=nd2.yentr;
         end;

       {if 1 of the 2 is a power, copy description}
 	   if ((nd1.classe=vcc) or (nd1.classe=vss)) then
         begin
 			n^.xalim:=nd1.xalim;
 			n^.yalim:=nd1.yalim;
         end;
 	   if ((nd2.classe=vcc) or (nd2.classe=vss)) then
         begin
 			n^.xalim:=nd2.xalim;
 			n^.yalim:=nd2.yalim;
         end;

       {concatenate the two connections lists}
 	   if (nd1.debutconnections=nil) then
 	 	 n^.debutconnections:=nd2.debutconnections
 	   else if (nd2.debutconnections=nil) then
         n^.debutconnections:=nd1.debutconnections
 	   else begin
 		 n^.debutconnections:=nd1.debutconnections;
 		 index:=nd1.debutconnections^.connsuiv;
 		 indexprecc:=nd1.debutconnections;
 		 while (index<>nil) do
           begin
 			 indexprecc:=index;
 			 index:=index^.connsuiv;
           end;
 		 indexprecc^.connsuiv:=nd2.debutconnections;
       end;

       {concatenate the two segments lists}
 	   if (nd1.debutsegments=nil) then
         n^.debutsegments:=nd2.debutsegments
 	   else if (nd2.debutsegments=nil) then
         n^.debutsegments:=nd1.debutsegments
 	   else
         begin
 		    n^.debutsegments:=nd1.debutsegments;
 		    WITH ND1 DO sindex:=debutsegments^.segsuiv;
 		    sindexprecc:=nd1.debutsegments;
 		    while (sindex<>nil) do
              begin
 				sindexprecc:=sindex;
 				sindex:=sindex^.segsuiv;
              end;
 		    sindexprecc^.segsuiv:=nd2.debutsegments;
         end;

       {and add the new segments to the list}
 	   if (npoints>1) then for i:=1 to npoints-1 do
 	     begin				{ segment i,i+1 }
 	       new(s);
 	       s^.xd:=tablepts[i].x; s^.yd:=tablepts[i].y;
 	       s^.xf:=tablepts[i+1].x; s^.yf:=tablepts[i+1].y;
 	       s^.segsuiv:=n^.debutsegments;
 	       n^.debutsegments:=s;
 	     end;

       {remplace each occurence of n2 with n1}
 	   if not(remplace(n2,n1)) then f:=false;

       {reconfigure the node array}
 	   noeuds[n1]:=n;
 	   noeuds[n2]:=nil;
 	   nbnoeud:=nbnoeud-1;
 	 end;

     {return value is true if ok}
     fusionne:=f;
   end;

 {--------------------- mouse definition of connections ------------------}

 procedure connecte;
   var x,y:integer;
       S:STRING;
       tpts:tablepnts;
       npts,bp:integer;
       noeuddepart,noeudarrivee:zonenoeud;
       ok:boolean;

   begin
     modifeffectuee;
     cr:=sacwk(fschema);
     cr:=ssplci(7);
     S:='CONNECTION';
     titre(s);
     repeat
       repeat
         S:='First node ?';
         info(S,WHITE);

         {get first connection point}
         bp:=litsouris(x,y);

         {check if node connected}
         if (bp=1) and (table[x,y].nconnect=0) then
           erreur (20);
       until (bp<>1) or (table[x,y].nconnect<>0);

       if (bp=1) then
 	   begin
 	     tpts[1].x:=x; tpts[1].y:=y; npts:=1;
 	     noeuddepart:=table[x,y].nconnect;
 	     repeat
           S:='Next node/pt ?';
           info(S,WHITE);

           {get second connection point}
 	       bp:=litsouris(x,y);

           {check if horizontal or vertical}
 	       ok:=((x=tpts[npts].x) or (y=tpts[npts].y));

           {connection not | or - ? If so, make it !}
 	       if not(ok) and (bp=1) then
             begin
               if abs(x-tpts[npts].x)>abs(y-tpts[npts].y) then
                 y:=tpts[npts].y
               else
                 x:=tpts[npts].x;
 			 end;

 	       if (bp=1) then
             begin

               {if so, draw the line and add it to the segments list}
               cr:=sacwk(fschema);
     		   cr:=ssplci(7);
 	           cr:=sln(xgrille(x),ygrille(y),
 			   xgrille(tpts[npts].x),ygrille(tpts[npts].y));
 			   npts:=succ(npts);
 			   tpts[npts].x:=x; tpts[npts].y:=y;
 			 end;
 	     until ((table[x,y].nconnect<>0) or (npts>=NBMPTS)
                        or (bp <> 1));

         {check final node number}
 	     noeudarrivee:=table[x,y].nconnect;

         {operation aborted ?}
         if (bp <> 1) then
           begin
             effacesegments(tpts,npts);
             bp:=1;
           end


         {no node at final position ? (-> too many segments) }
 	     else if noeudarrivee=0 then
           begin
 			 erreur(22);
 			 effacesegments(tpts,npts);
           end

         {same node than first one ?}
 	     else if noeuddepart=noeudarrivee then
 		   begin
             erreur(23);
             effacesegments(tpts,npts);
           end

         {else, try to combine the two nodes}
 	     else
           begin
             if not(fusionne(noeuddepart,noeudarrivee,tpts,npts)) then
 		       effacesegments(tpts,npts)
             else
               begin
                 sound(1000);
                 delay(50);
                 nosound;
               end;
           end;
 	   end;

       {redisplay network statistics}
       statistics;
     until (bp <> 1);
   end;

 {----------------------------- unit initialisation ----------------------}

 BEGIN
 END.

{==========================================================================}
{                              END OF FILE                                 }
{==========================================================================}

