{==========================================================================}
{ Program            : MOSSIM                                              }
{ Author             : Robert Lacoste, Chaville, France                    }
{ Version            : 1.3                                                 }
{ Release date       : 27/07/1996                                          }
{==========================================================================}

{--------------------------------------------------------------------------}
{                              UNIT CORRECTION                             }
{                                                                          }
{ Description : Management of network correction                           }
{                                                                          }
{--------------------------------------------------------------------------}

 UNIT CORRECT;                  {public declarations}

   INTERFACE
     USES GRAPH,
          DATA,
          GDS,
          PROCGEN,
          DESSINS,
          CONNECT;

     {interactive correction}
     PROCEDURE CORRECTION;

   IMPLEMENTATION              {local declarations}

   {--- erase a zone in the window view and erase the grid occupation -----}

   procedure effaceschema(x,y,mx,px,my,py:integer);
     var i,j:integer;
     begin

       {draw a black rectangle}
       cr:=ssfaci(0);
       cr:=sgr(xgrille(x-mx)-2,ygrille(y-my)-2,xgrille(x+px)+2,ygrille(y+py)+2);

       {for each grid point of this rectangle}
       for i:=x-mx to x+px do
 	     for j:=y-my to y+py do
 	       begin

             {frees the grid occupation record}
 	         with table[i,j] do begin
 				 element:=libre;
 				 nconnect:=0;
             end;

             {redraw the grid marker}
 	         CR:=smk(xgrille(i), ygrille(j));
 	       end;
     end;

   {-------- erase segments connecting a node to a given screen point -----}
   {warning : called recursively}

   procedure delie(numnd,x,y:integer);
     var ps,pst,pf,pft:ptseg;
 	     seul:boolean;
         t:tablepnts;

     begin

       {scan the segments list of this node}
       ps:=noeuds[numnd]^.debutsegments;
       pst:=nil; pf:=nil; pft:=nil;
       seul:=true;
       while (ps<>nil) do
 	     begin

           {check if current segment connected to the given screen point}
 	       if (((ps^.xd=x) and (ps^.yd=y)) or ((ps^.xf=x) and (ps^.yf=y)))
 	         then begin

               {store pointer to this segment}
 	           if (pst=nil) then
                 begin
                   pst:=ps;pft:=pf
                 end

               {check if more than one segment arrive to this point}
               else seul:=false;
 		     end;
 	       pf:=ps;
 	       ps:=ps^.segsuiv;
 	     end;

       {if one and only one segment was found...}
       if (seul and (pst<>nil)) then
 	     begin

           {...erase it !}
 	       t[1].x:=pst^.xd; t[1].y:=pst^.yd;
           t[2].x:=pst^.xf; t[2].y:=pst^.yf;
 	       effacesegments(t,2);

           {reconnect the linked structure}
 	       if (pft=nil) then noeuds[numnd]^.debutsegments:=pst^.segsuiv
 		   else pft^.segsuiv:=pst^.segsuiv;

           {if the other end of the deleted segment is not a junction,}
           {call the procedure recursively}

 	       if ((pst^.xd=x) and (pst^.yd=y)) then
 		     begin
 		       if (table[pst^.xf,pst^.yf].nconnect=0) then
 			     delie(numnd,pst^.xf,pst^.yf)
 		     end
 		   else begin
 		     if (table[pst^.xd,pst^.yd].nconnect=0) then
 			   delie(numnd,pst^.xd,pst^.yd);
 		   end;
 	     end;
     end;

   {------------------ disconnect a transistor from a given node ---------}

   procedure deconnecte(n,tr:integer);
     var pc,pd:ptconn;
 	     trouve:boolean;
     begin

       {scan the node's connection list}
       pc:=noeuds[n]^.debutconnections;
       if (pc<>nil) then
         begin
 	       pd:=nil;
 	       while (not(trouve) and (pc<>nil)) do
             begin

               {is this one conncected to the given transistor ?}
 	           trouve:=(pc^.num=tr);
 	           if not(trouve) then
                 begin
                   pd:=pc; pc:=pc^.connsuiv;
                 end;
 	         end;

           {at this point, it should be founded}
 	       if not(trouve) then erreur(94)
           else
 	         begin

               {reconstruct the linked list}
 	           if (pd=nil) then noeuds[n]^.debutconnections:=pc^.connsuiv
 			   else pd^.connsuiv:=pc^.connsuiv;

               {check if this node is now useless or not}
 	           if (noeuds[n]^.debutconnections=nil) then
 	             if (noeuds[n]^.classe=normal) then
 	               if ((noeuds[n]^.xpull=0) and (noeuds[n]^.ypull=0)) then
 	                 if (noeuds[n]^.numsortie=0) then
 			           begin
                         {if yes, free it}
 						 noeuds[n]:=nil;
 						 nbnoeud:=nbnoeud-1;
                       end;
 	         end;
        end;
     end;

   {-------------------- deleting of a transistor ------------------------}

   procedure supprtr(nt:integer);

     begin
       with transistors[nt]^do
 	     begin

           {erase graphicaly the component}
 	       effaceschema(trx,try,1,1,1,1);

           {erase all the segments connecting it to the network}
 	       case orientation of
 	         d: begin
 		          delie(grille,trx+1,try);
 		          delie(t1,trx-1,try-1);
 		          delie(t2,trx-1,try+1);
 	            end;
 	         g: begin
 		          delie(grille,trx-1,try);
 		          delie(t1,trx+1,try-1);
 		          delie(t2,trx+1,try+1);
                end;
 	         b: begin
 		          delie(grille,trx,try+1);
 		          delie(t1,trx-1,try-1);
 		          delie(t2,trx+1,try-1);
 	            end;
 	         h: begin
 		          delie(grille,trx,try-1);
 		          delie(t1,trx-1,try+1);
 		          delie(t2,trx+1,try+1);
 	            end;
 	       end;

         {deconnect the nodes to the transistor pins}
 	     deconnecte(grille,nt);
 	     deconnecte(t1,nt);
 	     deconnecte(t2,nt);
 	   end;

     {free the transistor record}
     transistors[nt]:=nil; nbtr:=nbtr-1;

     {redraw the segments}
     dessinniveaux(false,NOEUDS);
   end;

   {------------------------- deleting of an output ----------------------}

   procedure supprsort(n:integer);
     begin
       with noeuds[n]^ do
       begin

         {graphicaly erase the output}
 	     effaceschema(xsort,ysort,0,3,1,1);

         {disconnect it from the network}
 	     assignsorties[numsortie]:=0;
 	     delie(n,xsort,ysort);
 	     numsortie:=0;
 	     xsort:=0; ysort:=0;
       end;

       {check if node useless}
       if (noeuds[n]^.debutconnections=nil) then
         if (noeuds[n]^.classe=normal) then
 	       if ((noeuds[n]^.xpull=0) and (noeuds[n]^.ypull=0)) then
 	         if (noeuds[n]^.numsortie=0) then

               {if so, free it}
 			   begin
 				 noeuds[n]:=nil;
 				 nbnoeud:=nbnoeud-1;
               end;

       {redraw the segments}
       dessinniveaux(false,NOEUDS);
     end;

   {----------------- deleting of an input --------------------------------}

   procedure supprentr(n:integer);
     begin
       with noeuds[n]^ do
       begin

         {graphicaly erase the input}
 	     effaceschema(xentr,yentr,3,0,1,1);

         {disconnect it from the network}
      	 assignentrees[numentree]:=0;
 	     delie(n,xentr,yentr);
 	     numentree:=0;
 	     xentr:=0; yentr:=0; classe:=normal;
       end;

       {check if the node is useless}
       if (noeuds[n]^.debutconnections=nil) then
        if (noeuds[n]^.classe=normal) then
 	      if ((noeuds[n]^.xpull=0) and (noeuds[n]^.ypull=0)) then
 	        if (noeuds[n]^.numsortie=0) then

              {if so, free it}
 			  begin
 				noeuds[n]:=nil;
 				nbnoeud:=nbnoeud-1;
 			  end;

       {redraw the segments}
       dessinniveaux(false,NOEUDS);
     end;

   {--------------------- deleting of a pullup ---------------------------}

   procedure supprpull(n:integer);
     begin
       with noeuds[n]^ do
       begin

         {graphically erase the pullup}
 	     effaceschema(xpull,ypull,1,1,4,1);

         {disconnect it from the network}
 	     delie(n,xpull,ypull);
 	     xpull:=0; ypull:=0;
       end;

       {check if the node is useless}
       if (noeuds[n]^.debutconnections=nil) then
         if (noeuds[n]^.classe=normal) then
 	       if ((noeuds[n]^.xpull=0) and (noeuds[n]^.ypull=0)) then
 	         if (noeuds[n]^.numsortie=0) then
 				begin
 				  noeuds[n]:=nil;
 				  nbnoeud:=nbnoeud-1;
                end;

       {redraw the segments}
       dessinniveaux(false,NOEUDS);
     end;

   {-------------------- deleting of a power supply ----------------------}

   procedure suppralim(n:integer);
     begin
       with noeuds[n]^ do
       begin

         {graphicaly erase the power}
 	     if (classe=vcc) then effaceschema(xalim,yalim,1,1,3,0)
 			                  else effaceschema(xalim,yalim,2,2,0,2);

         {disconnect it from the network}
 	     delie(n,xalim,yalim);
 	     classe:=normal;
       end;

       {check if the node is useless}
       if (noeuds[n]^.debutconnections=nil) then
         if (noeuds[n]^.classe=normal) then
 	      if ((noeuds[n]^.xpull=0) and (noeuds[n]^.ypull=0)) then
 	        if (noeuds[n]^.numsortie=0) then

                      {if so, free it}
 			          begin
 						noeuds[n]:=nil;
 						nbnoeud:=nbnoeud-1;
 				      end;

       {redraw the segments}
       dessinniveaux(false,NOEUDS);
     end;

   {-------------------- deleting of a free text -------------------------}

   procedure supprtext(n:integer);
     begin
       with freetexts[n] do
       begin

         {graphicaly erase the text}
 	     effaceschema(x,y,0,(length(txt)*8) div ECART,1,0);

         {declare it free}
         x:=0; y:=0;
       end;

       {redraw the segments}
       dessinniveaux(false,NOEUDS);
     end;


   {--------------------- interactive correction of the network ----------}
   { only function implemented: deletion of a component}

   PROCEDURE CORRECTION;
   var x,y,bp:integer;
       s:string;

   begin
     modifeffectuee;
     S:='CORRECTION';titre(S);
     cr:=sacwk(fschema);

     repeat
       S:='What to delete?'; info(s,WHITE);

       {get the coordinates of the element to delete}
       bp:=litsouris(x,y);
       if ((x>0) and (x<=DIMMX) and (y>0) and (y<=DIMMY) and (bp=1)) then
         begin
           cr:=sacwk(fschema);

           {and delete it}
           case table[x,y].element of
 	         libre  : erreur(34);
 	         eltrans: supprtr(table[x,y].numtr);
 	         elpull : supprpull(table[x,y].numnoeud);
 	         elsort : supprsort(table[x,y].numnoeud);
 	         elentr : supprentr(table[x,y].numnoeud);
 	         elvcc,elvss: suppralim(table[x,y].numnoeud);
             eltxt  : supprtext(table[x,y].numtxt);
           end;
         end;
     until (bp <> 1);
   end;

   {--------------------------- unit initialisation ----------------------}

   BEGIN
   END.

{==========================================================================}
{                              END OF FILE                                 }
{==========================================================================}
