{==========================================================================}
{ Program            : MOSSIM                                              }
{ Author             : Robert Lacoste, Chaville, France                    }
{ Version            : 1.3                                                 }
{ Release date       : 27/07/1996                                          }
{==========================================================================}

{--------------------------------------------------------------------------}
{                              UNIT DATA                                   }
{                                                                          }
{ Description : Data type definitions and global variables declarations    }
{                                                                          }
{--------------------------------------------------------------------------}

 UNIT DATA;

  INTERFACE                    { public declarations }

  CONST
 	DIMX=640; DIMY=480;			{screen dimensions}
 	ECART=7;					{grid spacing}
 	ECARTHSIGN=16;				{horizontal signals spacing}
 	ECARTVSIGN=8;				{vertical signal spacing}
 	LONGSIGN=29;				{total number of time steps}

 	NBMTRANSIS=100;{<32768}		{maximum number of transistors}
 	NBMNOEUDS= 300;{<32768}		{maximum number of nodes}
 	NBMPTS   = 20;				{maximum number of segments per connections}

 	DIMMX=70; 					{grid dimensions}
 	DIMMY=46;

 	NBMENTREES=8;				{max number of inputs}
 	NBMSORTIES=8;				{max number of outputs}
 	NBMPATTES =(NBMENTREES+NBMSORTIES);

    LTEXT=10;                   {max length of free text strings}
    NBMTEXTS=30;                {max number of free text strings}

    fschema=1;                  {windows identifiers}
    fsignaux=2;
    ftexte=3;


  TYPE
 	coordx=0..DIMMX;			{grid indexes}
 	coordy=0..DIMMY;

 	zonenoeud= 0..NBMNOEUDS;    {node number}

    {enum types}
 	tristate=  (un,zero,nondef);                {signal level}
 	senstransi=(d,g,h,b,efface);			    {transistor orientation}
 	tclasse=   (entree,normal,vcc,vss,nul);		{node type}
 	typebroche=(tgr,tt1,tt2);			        {transistor pins}
 	typelement=(libre,eltrans,elpull,elsort,elentr,elvcc,elvss,eltxt);
                                                {grid position attribut}
 	tcurseur  =(croix,elastique);	   		    {mouse cursor type}
    typetransi=(pchannel,nchannel);             {transistor channel type}

    {transistor record}
 	trans = record
 		      orientation: senstransi;		    {orientation}
              channel:typetransi;               {channel type}
 		      trx: coordx;			            {screen position}
 		      try: coordy;
 		      grille,t1,t2: zonenoeud;  	    {connecting nodes}
 		    end;

    {connection pointer}
 	ptconn= ^connection;

    {connection record (one node include one linked list of connections)}
 	connection= record
 		          num : 0..NBMTRANSIS;		{transistor number}
 		          connsuiv:ptconn;			{next connection}
 		          broche:typebroche;		{transistor pin}
 		        end;

    {segment pointer}
 	ptseg= ^segmnt;

    {graphical segment record (line = linked list of segments)}
 	segmnt=    record
 		      xd:coordx;		         	{segment to draw}
 		      yd:coordy;
 		      xf:coordx;
 		      yf:coordy;
 		      segsuiv:ptseg;			    {next segment}
 		    end;

    {node record :                                                      }
    {  each node is defined :                                           }
    {  - Electricaly by its type (input, vss, vcc or normal),           }
    {    the list of transistors pins connected to it, presence or not  }
    {    of a pullup or an output pin, its current level                }
    {  - Graphicaly by the segments to draw for the connections, the    }
    {    screen positions of inpt, output, vss, vcc, pullup pins        }
    noeud= record
 		     classe: tclasse;			    {node type, or nul}
 		     numsortie:0..NBMSORTIES; 		{output number, or 0}
 		     xsort:coordx; ysort:coordy;	{output screen coordinates}
 		     debutconnections:ptconn;		{connections to this node}
 		     debutsegments:ptseg;			{segments to draw for this node}
 	         xpull:coordx; ypull:coordy;	{pullup corrdinates, or 0,0}
 		     etat: tristate;			    {current logic level}
 		     case tclasse of                {screen positions of elements}
 			    entree: (numentree:0..NBMENTREES;
 	                          xentr:coordx; yentr:coordy);
 			    vcc,vss:(xalim:coordx; yalim:coordy);
 			    normal: ();
           end;

    {pointers to transistors and nodes}
    pttransistor= ^trans;
 	ptnoeud=      ^noeud;

    {screen grid position occupation}
 	occupationcase=
             record
 			   element:typelement;		{symbol type here}
 			   nconnect:zonenoeud;		{node number if connected}
 			   case typelement of		{element number or 0}
 		             eltrans: (numtr:0..NBMTRANSIS);
 			         elpull,elsort,
 			         elentr,elvcc,elvss:(numnoeud:zonenoeud);
                     eltxt:(numtxt:0..NBMTEXTS);
 	                 libre: ();
 			 end;

    {screen coordinate record}
 	pnt=	 record
 			   x:coordx; y:coordy;
 			 end;

    {free text record}
    freetxt= record
               x:coordx; y:coordy;     {0,0 if not used}
               txt:string[LTEXT];
             end;

    {array types definitions}
 	tablepnts= array[1..NBMPTS] of pnt;
    tpnd= array[1..NBMNOEUDS] of ptnoeud;
    tptr= array[1..NBMTRANSIS] of pttransistor;
    tptb= packed array[coordx,coordy] of occupationcase;
    tpve= array[1..NBMENTREES,1..LONGSIGN] of 0..1;
    tpvs= array[1..NBMSORTIES,1..LONGSIGN] of tristate;
    tpae= array[1..NBMENTREES] of zonenoeud;
    tpas= array[1..NBMSORTIES] of zonenoeud;

    {bit manipulating definitions}
    BITS=0..7;
    BYTE=SET OF BITS;


 	{global variables declarations}
    VAR
       cr,i,j:integer;		    {gds procedures codes}

       numfichier: 0..9999;	    {current file number}
       modifie: boolean;	    {current file modified since save ?}
       signaff:boolean;		    {current signals drawed on network ?}
       nbnoeud: 0..NBMNOEUDS;	{current number of nodes}
       nbtr:0..NBMTRANSIS;	    {   "      "       transistors }

       noeuds: tpnd;	        {array of pointers to networks nodes}
       transistors:tptr;	    {array of pointers to network transistors }

       table: tptb;             {screen grid occupation array}

       valentrees:tpve;         {inputs levels array}
       valsorties:tpvs;         {outputs levls array}

       assignentrees:tpae;	    {inputs connecting nodes, or 0}
       assignsorties:tpas;      {output connecting nodes, or 0}

       freetexts: array[1..NBMTEXTS] of freetxt; {free texts}

       fini: boolean;		    {for main menu}

       sortiescalculees:boolean;{output signals displayed ?}

       mainheapmark:POINTER;    {heap marker for memory cleaning}

     {data initialisations}
     PROCEDURE INITIALISATIONS;

   IMPLEMENTATION               {private declarations}

   {----------------------- data initialisations --------------------------}

   PROCEDURE INITIALISATIONS;
   var i,j:integer;
   begin

     {release all allocated memory}
     release (mainheapmark);
     mark (mainheapmark);

     {put all global variables to init value}
     numfichier:=9999; modifie:=false;
     nbnoeud:=0; nbtr:=0; signaff:=false;
     sortiescalculees:=false;
     for i:=1 to NBMNOEUDS  do noeuds[i]:=nil;
     for i:=1 to NBMTRANSIS do transistors[i]:=nil;
     for i:=1 to DIMMX do for j:=1 to DIMMY do with table[i,j] do
 			begin
 			  element:=libre;
 			  nconnect:=0;
 			end;
     for i:=1 to NBMENTREES do assignentrees[i]:=0;
     for i:=1 to NBMSORTIES do for j:=1 to LONGSIGN do valsorties[i,j]:=nondef;
     for i:=1 to NBMENTREES do for j:=1 to LONGSIGN do valentrees[i,j]:=0;
     for i:=1 to NBMSORTIES do assignsorties[i]:=0;
     for i:=1 to NBMTEXTS do
       begin
         freetexts[i].x:=0; freetexts[i].y:=0;
       end;
   end;


   BEGIN
   END.

{==========================================================================}
{                              END OF FILE                                 }
{==========================================================================}
