{==========================================================================}
{ Program            : MOSSIM                                              }
{ Author             : Robert Lacoste, Chaville, France                    }
{ Version            : 1.3                                                 }
{ Release date       : 27/07/1996                                          }
{==========================================================================}

{--------------------------------------------------------------------------}
{                              UNIT DESSINS                                }
{                                                                          }
{ Description : Drawing of the different components                        }
{                                                                          }
{--------------------------------------------------------------------------}

 UNIT DESSINS;


 INTERFACE                       {public declarations}
   USES GRAPH,
        DATA,
        PROCGEN,
        GDS;

   {drawing of network connections}
   PROCEDURE DESSINNIVEAUX(V:BOOLEAN; VAR NOEUDS:TPND);

   {drawing of a transistor}
   PROCEDURE DESSINTRANSI(X,Y:INTEGER;T:SENSTRANSI;CHAN:TYPETRANSI);

   {drawing of a pullup}
   PROCEDURE DESSINPULLUP(X,Y:INTEGER);

   {drawing of powers}
   PROCEDURE DESSINVCC(X,Y:INTEGER);
   PROCEDURE DESSINVSS(X,Y:INTEGER);

   {drawing of inputs and outputs}
   PROCEDURE DESSINENTREE(X,Y,NUM:INTEGER);
   PROCEDURE DESSINSORTIE(X,Y,NUM:INTEGER);

   {drawing of a free text}
   PROCEDURE DESSINTEXTE(X,Y:INTEGER;TEXTE:STRING);

   {drawing of the grid}
   PROCEDURE AFFICHEGRILLE;

 IMPLEMENTATION                  {local declarations}


   {----------------------- drawing of network connections ----------------}
   { if V=false, all connections grey, if V=true, connection color }
   { depending on node electrical state }

   PROCEDURE DESSINNIVEAUX   {(V:BOOLEAN; VAR NOEUDS:TPND)};
   var i:integer;
       s:ptseg;
   begin
     cr:=sacwk(fschema);

     {for each nodes}
     for i:=1 to NBMNOEUDS do if (noeuds[i]<>nil) then
       begin

         {select segment color}
         if not v then
           cr:=ssplci(7)
         else
           case noeuds[i]^.etat of
             un :    cr:=ssplci(RED);
             zero :  cr:=ssplci(LIGHTBLUE);
             nondef :cr:=ssplci(LIGHTGRAY);
           end;

         {draw each segment}
 	     s:=noeuds[i]^.debutsegments;
 	     while (s<>nil) do
 	      begin
           cr:=sln(XGRILLE(s^.xd),YGRILLE(s^.yd),
                   XGRILLE(s^.xf),YGRILLE(s^.yf));
 	       s:=s^.segsuiv;
 	      end;
       end;
   end;

   {--------------------- draw a transistor ------------------------------}

   PROCEDURE DESSINTRANSI    {(X,Y:INTEGER;T:SENSTRANSI;CHAN:TYPETRANSI)};
   var xr,yr,xrm,xrp,yrm,yrp:integer;
   begin
     xr:=XGRILLE(x); yr:=YGRILLE(y);
     xrm:=xr-ECART; xrp:=xr+ECART; yrm:=yr-ECART; yrp:=yr+ECART;
     cr:=sacwk(fschema);
     cr:=ssplci(13);
     case t of
       d:begin       {right sided}
 	      cr:=sln(xr-1,yrm-2,xr-1,yrp+2);
 	      cr:=sln(xr+1,yrm-2,xr+1,yrp+2);
          case chan of
            nchannel:
              cr:=sln(xr+2,yr,xrp,yr);
            pchannel:
              begin
                cr:=sln(xr+4,yr,xrp,yr);
                cr:=sln(xr+4,yr-2,xr,yr-2);
                cr:=sln(xr+4,yr+2,xr,yr+2);
                cr:=sln(xr+4,yr-2,xr+4,yr+2);
              end;
          end;
          cr:=sln(xrm,yrm,xr-2,yrm);
 	      cr:=sln(xrm,yrp,xr-2,yrp);
 	      CR:=SSPLCI(0); CR:=SLN(XR,YRM,XR,YRP);
          cr:=smk2(xrm,yrm); cr:=smk2(xrm,yrp); cr:=smk2(xrp,yr);
 	    end;
       g:begin       {left sided}
 	      cr:=sln(xr+1,yrm-2,xr+1,yrp+2);
 	      cr:=sln(xr-1,yrm-2,xr-1,yrp+2);
          case chan of
            nchannel:
              cr:=sln(xr-2,yr,xrm,yr);
            pchannel:
              begin
                cr:=sln(xr-4,yr,xrm,yr);
                cr:=sln(xr-4,yr-2,xr,yr-2);
                cr:=sln(xr-4,yr+2,xr,yr+2);
                cr:=sln(xr-4,yr-2,xr-4,yr+2);
              end;
          end;
          cr:=sln(xrp,yrm,xr+2,yrm);
 	      cr:=sln(xrp,yrp,xr+2,yrp);
 	      CR:=SSPLCI(0); CR:=SLN(XR,YRM,XR,YRP);
          cr:=smk2(xrp,yrm); cr:=smk2(xrp,yrp); cr:=smk2(xrm,yr);
 	    end;
       h:begin       {up sided}
 	      cr:=sln(xrm-2,yr-1,xrp+2,yr-1);
 	      cr:=sln(xrm-2,yr+1,xrp+2,yr+1);
 	      case chan of
            nchannel:
              cr:=sln(xr,yrm,xr,yr-2);
            pchannel:
              begin
                cr:=sln(xr,yrm,xr,yr-4);
                cr:=sln(xr-2,yr-4,xr-2,yr-2);
                cr:=sln(xr+2,yr-4,xr+2,yr-2);
                cr:=sln(xr-2,yr-4,xr+2,yr-4);
              end;
          end;
          cr:=sln(xrm,yrp,xrm,yr+2);
          cr:=sln(xrp,yrp,xrp,yr+2);
 	      CR:=SSPLCI(0); CR:=SLN(XRM,YR,XRP,YR);
          cr:=smk2(xr,yrm); cr:=smk2(xrm,yrp); cr:=smk2(xrp,yrp);
 	    end;
       b:begin       {down sided}
 	      cr:=sln(xrm-2,yr+1,xrp+2,yr+1);
 	      cr:=sln(xrm-2,yr-1,xrp+2,yr-1);
 	      case chan of
            nchannel:
              cr:=sln(xr,yrp,xr,yr+2);
            pchannel:
              begin
                cr:=sln(xr,yrp,xr,yr+4);
                cr:=sln(xr-2,yr+4,xr-2,yr+2);
                cr:=sln(xr+2,yr+4,xr+2,yr+2);
                cr:=sln(xr-2,yr+4,xr+2,yr+4);
              end;
          end;
 	      cr:=sln(xrm,yrm,xrm,yr-2);
          cr:=sln(xrp,yrm,xrp,yr-2);
 	      CR:=SSPLCI(0); CR:=SLN(XRM,YR,XRP,YR);
          cr:=smk2(xr,yrp); cr:=smk2(xrm,yrm); cr:=smk2(xrp,yrm);
 	    end;
       efface:;
     end;
   end;

   {---------------------- drawing of a pullup ---------------------------}

   PROCEDURE DESSINPULLUP    {(X,Y:INTEGER)};
   var xp,yp:integer;
   begin
     cr:=sacwk(fschema);
     xp:=XGRILLE(x); yp:=YGRILLE(y);
     cr:=ssplci(14);
     cr:=ssfaci(2);
     cr:=sgr(xp-2,yp-ECART,xp+2,yp-2*ECART-1);
     cr:=sln(xp,yp,xp,yp-ECART+1);
     cr:=sln(xp,yp-2*ECART-2,xp,yp-3*ECART+1);
     cr:=smk2(xp,yp);
     cr:=ssplci(14); yp:=yp-4*ECART+1;
     cr:=sln(xp,yp-2,xp,yp+ 2);
     cr:=sln(xp-2,yp,xp+2,yp);
   end;

   {--------------------- drawing of a vcc -------------------------------}

   PROCEDURE DESSINVCC       {(X,Y:INTEGER)};
   var xr,yr:integer;
   begin
     cr:=sacwk(fschema);
     xr:=XGRILLE(x); yr:=YGRILLE(y);
     cr:=ssplci(14);
     cr:=sln(xr,yr,xr,yr-ECART+1);
     cr:=smk2(xr,yr); yr:=yr-2*ECART+1;
     cr:=ssplci(14);
     cr:=sln(xr-2,yr,xr+2,yr);
     cr:=sln(xr,yr-2,xr,yr+2);
   end;

   {------------------------- drawing of a vss ----------------------------}

   PROCEDURE DESSINVSS       {(X,Y:INTEGER)};
   var xr,yr:integer;
   begin
     cr:=sacwk(fschema);
     xr:=XGRILLE(x); yr:=YGRILLE(y);
     cr:=ssplci(14);
     cr:=sln(xr,yr,xr,yr+ECART);
     cr:=smk2(xr,yr); yr:=yr+ECART; cr:=ssplci(6);
     cr:=sln(xr-ECART+1,yr,xr+ECART-1,yr);
     cr:=sln(xr-ECART+3,yr+2,xr+ECART-3,yr+2);
     cr:=sln(xr-ECART+5,yr+4,xr+ECART-5,yr+4);
   end;

   {----------------------- drawing of an input ---------------------------}

   PROCEDURE DESSINENTREE    {(X,Y,NUM:INTEGER)};
   var xr,yr:integer; ss:string;
   begin
     cr:=sacwk(fschema); xr:=XGRILLE(x); yr:=YGRILLE(y);
     cr:=ssplci(WHITE);
     cr:=sln(xr,yr,xr-ECART-2,yr);
     cr:=sln(xr,yr,xr-2,yr-2); cr:=sln(xr,yr,xr-2,yr+2);
     cr:=smk2(xr,yr); xr:=xr-ECART-2;
     cr:=sln(xr,yr+6,xr,yr-6); cr:=sln(xr,yr+6,xr-10,yr+6);
     cr:=sln(xr,yr-6,xr-10,yr-6); cr:=sln(xr-10,yr-6,xr-10,yr+6);
     CR:=SSPLCI(0); CR:=SLN(XR-1,YR-4,XR-9,YR-4);
     CHDIG(num,ss);
     cr:=stx(xr-8,yr-3,LIGHTCYAN,ss);
   end;

   {--------------------- drawing of an output ----------------------------}

   PROCEDURE DESSINSORTIE    {(X,Y,NUM:INTEGER)};
   var xr,yr:integer; ss:string;
   begin
     cr:=sacwk(fschema); xr:=XGRILLE(x); yr:=YGRILLE(y);
     cr:=ssplci(WHITE);
     cr:=sln(xr,yr,xr+ECART+2,yr);
     cr:=smk2(xr,yr); xr:=xr+ECART+2;
     cr:=sln(xr,yr,xr-2,yr+2); cr:=sln(xr,yr,xr-2,yr-2);
     cr:=sln(xr,yr+6,xr+10,yr+6); cr:=sln(xr,yr-6,xr+10,yr-6);
     cr:=sln(xr,yr-6,xr,yr+6); cr:=sln(xr+10,yr-6,xr+10,yr+6);
     CR:=SSPLCI(0); CR:=SLN(XR+1,YR-4,XR+9,YR-4);
     CHDIG(num,ss);
     cr:=stx(xr+2,yr-3,LIGHTGREEN,ss);
   end;

   {---------------------- drawing of a free text -------------------------}

   PROCEDURE DESSINTEXTE{(X,Y:INTEGER;TEXTE:STRING)};
   var xr,yr:integer;
       lmax:integer;
       s:string;
   begin
     cr:=sacwk(fschema); xr:=XGRILLE(x); yr:=YGRILLE(y);
     lmax:=((DIMMX-x)*ECART) div 8 + 1;
     s:=copy(texte,1,lmax);
     cr:=stx(xr,yr-8,CYAN,s);
   end;

   {--------------------------- grid drawing ------------------------------}

   PROCEDURE AFFICHEGRILLE;
   var i,j:integer;
   begin
     cr:=sacwk(fschema);cr:=sclrwk;
     FOR I:=1 TO DIMMX DO FOR J:=1 TO DIMMY DO CR:=SMK(XGRILLE(I),YGRILLE(J));
   end;

 {----------------------------- unit initialisation -----------------------}

 BEGIN
 END.

{==========================================================================}
{                              END OF FILE                                 }
{==========================================================================}
