{==========================================================================}
{ Program            : MOSSIM                                              }
{ Author             : Robert Lacoste, Chaville, France                    }
{ Version            : 1.3                                                 }
{ Release date       : 27/07/1996                                          }
{==========================================================================}

{--------------------------------------------------------------------------}
{                              UNIT FICH                                   }
{                                                                          }
{ Description : Management of disk store and recall of a network           }
{                                                                          }
{--------------------------------------------------------------------------}

{++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
File structure :
--------------

File name : "Fnnnn.MOS"

Type of file : FILE OF INT

Structure :

  <Begin of file>
  <For each transistor>
    -1
    transistor number
    orientation
    channel (n or p)
    x,y
    grid node
    t1 node
    t2 node
  <For each free text>
    -2
    freetext number
    x,y
    char(1)...char(n)
    0
  <For each node>
    -3
    class
    output number (or 0)
    x,y output (or 0)
    x,y pullup (or 0)
    <if class=input>
      input number
      x,y input
    <if class=power>
      x,y power
    <For each connection of this node>
      -31
      transistor number
      pin
    <For each segment of this node>
      -32
      x1,y1
      x2,y2
  <for each input>
    -4
    input number
    val1, val2, ..., valn (0/1)
  -5
  <End of file>

+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

 UNIT FICH;                     {public declarations}

 INTERFACE
   USES GRAPH,
        CRT,
        DATA,
        GDS,
        DESSINS,
        PROCGEN,
        AJOUTS;

   {check if a given file exist}
   FUNCTION FICHIEREXISTE(S:STRING):BOOLEAN;

   {save the current network}
   PROCEDURE SAUVEGARDE;

   {reload a network}
   PROCEDURE CHARGEMENT;


 IMPLEMENTATION                 {private declarations}

   TYPE FILEINT = FILE OF INTEGER;

   VAR F:FILEINT;
       SZ:STRING;
       a:integer;

   {------------------------ check if a given file exist -----------------}
   { (leave the file open) }

   FUNCTION FICHIEREXISTE{(S:STRING):BOOLEAN};
     VAR SS:STRING; B:BOOLEAN;
     BEGIN

       {construct the file name}
       SS:=CONCAT(S,'.MOS');

       {try to open it with error trapping off}
       (*$I-*)
       ASSIGN(F,SS);
       RESET(F);
       B:=(IORESULT=0);
       (*$I+*)

       {return the result}
       FICHIEREXISTE:=B;
     END;

   {------------------- save the current network --------------------------}

   PROCEDURE SAUVEGARDE;
   var i,j,a:integer;
       ok:boolean;
       S:STRING;
       pcomp:ptconn;
       psegm:ptseg;
       name:string;
   begin
     SZ:='NETWORK SAVE';titre(SZ);

     {get file name}
     littxt(name,'Name',8,['a'..'z','A'..'Z','0'..'9','-','_']);
     ok:=true;
     if length(Name)>0 then
      begin
       {if file already exist, ask for a confirmation}
       if fichierexiste(name) then ok:=confirme;
        if ok then
        begin
     	 SZ:='WORKING...';info(SZ,WHITE);
         modifie:=false;

         {open the file for writing}
         rewrite(f);

         {save each transistor}
 	     i:=1;
 	     while (i<=NBMTRANSIS) do
 	       begin
 	         if (transistors[i]<>nil) then
               with transistors[i]^ do
 		         begin
 	               a:=-1; write(f,a);
 	               write(f,i);
 		           a:=ord(orientation); write(f,a);
                   a:=ord(channel); write(f,a);
 		           a:=trx; write(f,a);
 		           a:=try; write(f,a);
 		           a:=grille; write(f,a);
 		           a:=t1; write(f,a);
 		           a:=t2; write(f,a);
 		         end;
 	         i:=succ(i);
 	       end;

         {save each freetext}
 	     i:=1;
 	     while (i<=NBMTEXTS) do
 	       begin
 	         if (freetexts[i].x<>0) or (freetexts[i].y<>0) then
               with freetexts[i] do
 		         begin
 	               a:=-2; write(f,a);
 	               write(f,i);
 		           a:=x; write(f,a);
 		           a:=y; write(f,a);
                   for j:=1 to length(txt) do
                     begin
 		               a:=ord(txt[j]); write(f,a);
                     end;
                   a:=0; write(f,a);
 		         end;
 	         i:=succ(i);
 	       end;

         {save each node}
 	     i:=1;
 	     while (i<=NBMNOEUDS) do
 	       begin
 	         if (noeuds[i]<>nil) then
 	           with noeuds[i]^ do
 	             begin
 		           a:=-3; write(f,a);
 	               write(f,i);
 		           a:=ord(classe); write(f,a);
 		           a:=numsortie; write(f,a);
 		           a:=xsort; write(f,a);
 		           a:=ysort; write(f,a);
 		           a:=xpull; write(f,a);
 		           a:=ypull; write(f,a);
 		           if (classe=entree) then begin
 					  a:=numentree; write(f,a);
 					  a:=xentr; write(f,a);
 					  a:=yentr; write(f,a);
 				   end;
 		           if ((classe=vcc) or (classe=vss)) then begin
 					  a:=xalim; write(f,a);
                      a:=yalim; write(f,a);
                   end;

                   {save each connection of this node}
 	               pcomp:=debutconnections;
 		           while (pcomp<>nil) do
 		             begin
 		               a:=-31; write(f,a);
 		               a:=pcomp^.num; write(f,a);
 		               a:=ord(pcomp^.broche); write(f,a);
 		               pcomp:=pcomp^.connsuiv;
 		             end;

                   {save each segment of this node}
 		           psegm:=debutsegments;
 		           while (psegm<>nil) do
 		             begin
 		               a:=-32; write(f,a);
 		               a:=psegm^.xd; write(f,a);
 		               a:=psegm^.yd; write(f,a);
 		               a:=psegm^.xf; write(f,a);
 		               a:=psegm^.yf; write(f,a);
 		               psegm:=psegm^.segsuiv;
 		             end;
 	             end;
 	         i:=succ(i);
 	       end;

         {save each input signal}
 	     for i:=1 to NBMENTREES do
 	       begin
 	         a:=-4; write(f,a);
 	         write(f,i);
             for j:=1 to LONGSIGN do
               begin
 		         a:=valentrees[i,j]; write(f,a);
		       end;
 	       end;

         {write file terminator}
         a:=-5; write(f,a);
 	     close(f);
       end;
      end;
   end;

   {------------------------ restoring of transistors -------------------}

   PROCEDURE LITTRANSIS(VAR F:FILEINT);
     var i:integer;
       ss: senstransi;
       numtransi:integer;
       x,y:integer;
       ptr:pttransistor;
       ptn:ptnoeud;
       clnoeud:integer;
       s:ptseg; c:ptconn;
       chan:typetransi;

     BEGIN
        {transistor starting flag ?}
 	    while (a=-1) do
 	    begin

          {restore transistor caracteristics}
 	      read(f,a); numtransi:=a;
 	      read(f,a); case a of
 			0: ss:=d;
 			1: ss:=g;
 			2: ss:=h;
 			3: ss:=b;
 		      end;
          read(f,a); chan:=typetransi(a);
 	      read(f,a); x:=a; read(f,a); y:=a;

          {create and draw this transistor}
 	      new(ptr);
 	      transistors[numtransi]:=ptr;
 	      dessintransi(x,y,ss,chan);
 	      nbtr:=nbtr+1;
 	      ptr^.orientation:=ss;
          ptr^.channel:=chan;
 	      ptr^.trx:=x; ptr^.try:=y;

          {read transistor connections}
 	      read(f,a); ptr^.grille:=a;
 	      read(f,a); ptr^.t1:=a;
 	      read(f,a); ptr^.t2:=a;

          {read next integer}
 	      read(f,a);

          {assign grid occupation array}
          mettable(x,y,1,1,1,1,eltrans,numtransi);
 		  mettrtable(ss,x,y,ptr);
        end;
     END;

   {------------------------ restoring of freetexts --------------------}

   PROCEDURE LITTEXTS(VAR F:FILEINT);
     var i:integer;
       n:integer;
       s:string;

     BEGIN
        {freetext starting flag ?}
 	    while (a=-2) do
 	    begin

          {restore text caracteristics}
 	      read(f,a); n:=a;
 	      read(f,a); freetexts[n].x:=a; read(f,a); freetexts[n].y:=a;
          freetexts[n].txt:='';
          read(f,a); i:=0;
          while ((a>0) and (i<LTEXT)) do
            begin
              i:=i+1;
              s:='?'; s[1]:=chr(a);
              freetexts[n].txt:=concat(freetexts[n].txt,s);
              read(f,a);
            end;

          {create and draw this text}
 	      dessintexte(freetexts[n].x,freetexts[n].y,freetexts[n].txt);

          {read next integer}
 	      read(f,a);

          {assign grid occupation array}
          mettable(freetexts[n].x,freetexts[n].y,0,0,0,0,eltxt,n);
        end;
     END;


   {----------------------- restoring of nodes ---------------------------}

   PROCEDURE LITNOEUDS(VAR F1:FILEINT);
   var i:INTEGER;
       ss: senstransi;
       numtransi:integer;
       x,y:integer;
       ptr:pttransistor;
       ptn:ptnoeud;
       clnoeud:integer;
       s:ptseg; c:ptconn;
       noeud:integer;

    BEGIN
      {node starting flag ?}
 	  while (a=-3) do
 	    begin

          {restore node caracteristics}
 	      read(f,a); noeud:=a;
 	      read(f,a); clnoeud:=a;

          {create the node}
 	      nbnoeud:=nbnoeud+1;
 	      new(ptn); noeuds[noeud]:=ptn;

          {restore and assign the node output parameters}
 	      read(f,a); ptn^.numsortie:=a;
 	      read(f,a); ptn^.xsort:=a; read(f,a); ptn^.ysort:=a;
 	      if (ptn^.numsortie<>0) then
 			begin

              {if this node is an output, draw it}
 			  dessinsortie(ptn^.xsort,ptn^.ysort, ptn^.numsortie);
 			  mettable(ptn^.xsort,ptn^.ysort,0,4,1,1,elsort,noeud);
 			  table[ptn^.xsort,ptn^.ysort].nconnect:=noeud;
 			  assignsorties[ptn^.numsortie]:=noeud;
 			end;

          {restore and assign the node pullup parameters}
 	      read(f,a); ptn^.xpull:=a; read(f,a); ptn^.ypull:=a;
 	      if (ptn^.xpull<>0) then
 			begin

              {if this node is a pullup, draw it}
 			  dessinpullup(ptn^.xpull,ptn^.ypull);
 			  mettable(ptn^.xpull,ptn^.ypull,1,1,5,1,elpull,noeud);
 			  table[ptn^.xpull,ptn^.ypull].nconnect:=noeud;
 			end;

          {initialise node connections and segments lists}
 	      with ptn^ do begin
 			     xalim:=0; yalim:=0; debutconnections:=nil;
 			     debutsegments:=nil; etat:=nondef;
 			   end;

          {restore node complementary parameters if vcc, vss or input}
 	      case clnoeud of
   		    0: begin
 		         ptn^.classe:=entree;
 		         read(f,a); ptn^.numentree:=a;
 		         read(f,a); ptn^.xentr:=a; read(f,a); ptn^.yentr:=a;
 		         assignentrees[ptn^.numentree]:=noeud;
 		         dessinentree(ptn^.xentr,ptn^.yentr,ptn^.numentree);
 		         mettable(ptn^.xentr,ptn^.yentr,4,0,1,1,elentr,noeud);
 		         table[ptn^.xentr,ptn^.yentr].nconnect:=noeud;
 		       end;
 		    1: ptn^.classe:=normal;
 		    2: begin
 		         ptn^.classe:=vcc;
 		         read(f,a); ptn^.xalim:=a; read(f,a); ptn^.yalim:=a;
 		         dessinvcc(ptn^.xalim,ptn^.yalim);
 		         mettable(ptn^.xalim,ptn^.yalim,1,1,3,0,elvcc,noeud);
 		         table[ptn^.xalim,ptn^.yalim].nconnect:=noeud;
 		       end;
 		    3: begin
 		         ptn^.classe:=vss;
 		         read(f,a); ptn^.xalim:=a; read(f,a); ptn^.yalim:=a;
 		         dessinvss(ptn^.xalim,ptn^.yalim);
 		         mettable(ptn^.xalim,ptn^.yalim,2,2,0,2,elvss,noeud);
 		        table[ptn^.xalim,ptn^.yalim].nconnect:=noeud;
 		      end;
 		  end;

          {restore the node connections list}
 	      read(f,a);
 	      while (a=-31) do begin
 				  new(c);
 				  read(f,a); c^.num:=a; read(f,a);
 				  case a of
 					   0: c^.broche:=tgr;
 					   1: c^.broche:=tt1;
 					   2: c^.broche:=tt2;
 				  end;
  				  c^.connsuiv:=ptn^.debutconnections;
 				  ptn^.debutconnections:=c; read(f,a);
 				end;

          {restore the node segments list}
 	      while (a=-32) do begin
 				  new(s);
 				  read(f,a); s^.xd:=a; read(f,a); s^.yd:=a;
 				  read(f,a); s^.xf:=a; read(f,a); s^.yf:=a;
 				  s^.segsuiv:=ptn^.debutsegments;
 				  ptn^.debutsegments:=s; read(f,a);
 				end;
       END;
   END;

   {------------------------ restoring of INPUTS VALUES ------------------}

   PROCEDURE LITENTREES(VAR F:FILEINT);
     var i,j:integer;
       n:integer;

     BEGIN
        {inputs starting flag ?}
 	    while (a=-4) do
 	    begin

          {restore input number}
 	      read(f,a); n:=a;
          read(f,a);

          i:=1;
          if (n>0) and (n<=NBMENTREES) then
            while ((a>=0) and (i<=LONGSIGN)) do
            begin
              if a=0 then valentrees[n,i]:=0
                     else valentrees[n,i]:=1;
              i:=i+1;
              read(f,a);
            end;
        end;
     END;


   {---------------------- restoring of a network -------------------------}

   PROCEDURE CHARGEMENT;
   var i,j:integer;
       ok:boolean;
       sn:string;
       name:string;

   begin
     SZ:='NETWORK LOAD';titre(SZ);
     ok:=true;

     {check if a confirmation is needed}
     if modifie then ok:=confirme;
     if ok then
       begin
         {ask of file name}
 	     littxt(name,'Name',8,['a'..'z','A'..'Z','0'..'9','-','_']);
         if length(Name)>0 then
 	      if not(fichierexiste(name)) then erreur(5) else
 	       begin
        	 SZ:='Working...'; info(SZ,WHITE);

             {reset the network in memory}
 	         initialisations;

             {redraw the grid}
 	         affichegrille;

             {and read the file...}
             reset(f); read(f,a);
 	         LITTRANSIS(F);
             LITTEXTS(F);
             LITNOEUDS(F);
             LITENTREES(F);

             {check if end of file identifier is correct}
 	         if (a<>-5) then erreur(40);

             {close file}
 	         close(f);

             {and draw the segments}
 	         dessinniveaux(false,NOEUDS);
 	       end;
       end;
   end;

 {----------------------------- unit initialisation -----------------------}

 BEGIN
 END.

{==========================================================================}
{                              END OF FILE                                 }
{==========================================================================}
