{==========================================================================}
{ Program            : MOSSIM                                              }
{ Author             : Robert Lacoste, Chaville, France                    }
{ Version            : 1.3                                                 }
{ Release date       : 27/07/1996                                          }
{==========================================================================}

{--------------------------------------------------------------------------}
{                              UNIT GDS                                    }
{                                                                          }
{ Description : Very simplified emulation of the original Bull GDS         }
{               graphic library                                            }
{                                                                          }
{--------------------------------------------------------------------------}

 UNIT GDS;

   INTERFACE                    {Public declarations}

     {active window selection}
     FUNCTION SACWK(N:INTEGER):INTEGER;

     {draw a line}
     FUNCTION SLN(X1,Y1,X2,Y2:INTEGER):INTEGER;

     {select line color}
     FUNCTION SSPLCI(N:INTEGER):INTEGER;

     {erase active window}
     FUNCTION SCLRWK:INTEGER;

     {draw a box}
     FUNCTION SGR(X1,Y1,X2,Y2:INTEGER):INTEGER;

     {select box filling color}
     FUNCTION SSFACI(N:INTEGER):INTEGER;

     {draw a text string}
     FUNCTION STX(X,Y,COL:INTEGER; S:STRING):INTEGER;

     {draw a pixel}
     FUNCTION SMK(X,Y:INTEGER):INTEGER;

     {draw a fat pixel}
     FUNCTION SMK2(X,Y:INTEGER):INTEGER;

     {initialise graphic mode}
     PROCEDURE OUVREFENETRES;

     {close graphic mode}
     PROCEDURE FERMEFENETRES;



   IMPLEMENTATION                {private declarations}

    USES GRAPH,DATA;

    {local static variables}
    VAR FACTUELLE,FDX,FDY,CLIGNE,CFACI:INTEGER;

    {-------------------- select active window ---------------------------}

    FUNCTION SACWK {(N:INTEGER):INTEGER} ;
       BEGIN
         {store window number and define offsets}
         CASE N OF
           FSCHEMA: BEGIN                        {main network window}
                      FDX:=0; FDY:=0;
                    END;
           FSIGNAUX:BEGIN                        {signals window}
                      FDX:=0; FDY:=332;
                    END;
           FTEXTE:  BEGIN                        {text window}
                      FDX:=500; FDY:=0;
                    END;
         END;
         FACTUELLE:=N;
       END;

    {--------------------- draw a line with current color ------------------}

    FUNCTION SLN   {(X1,Y1,X2,Y2:INTEGER):INTEGER} ;
       BEGIN
         SETCOLOR(CLIGNE);
         LINE(X1+FDX,Y1+FDY,X2+FDX,Y2+FDY);
       END;

    {--------------------------- define line color -------------------------}

    FUNCTION SSPLCI  {(N:INTEGER):INTEGER};
       BEGIN
         CLIGNE:=N;
       END;

    {--------------------------- clear active window -----------------------}

    FUNCTION SCLRWK  {:integer};
       BEGIN
         SETCOLOR(BLACK);
         SETFILLSTYLE(1,BLACK);
         CASE FACTUELLE OF
           FSCHEMA: BEGIN
                      BAR(1,1,498,330);
                    END;
           FSIGNAUX:BEGIN
                      BAR(26,334,498,478);
                    END;
           FTEXTE:  BEGIN
                      BAR(502,2,638,478);
                    END;
         END;
       END;

    {---------------- draw a box with current paint color ------------------}

    FUNCTION SGR   {(X1,Y1,X2,Y2:INTEGER):INTEGER} ;
       VAR A:INTEGER;
       BEGIN
         SETFILLSTYLE(1,CFACI);
         BAR(X1+FDX,Y1+FDY,X2+FDX,Y2+FDY);
       END;

    {-------------------- select surrent paint color -----------------------}

    FUNCTION SSFACI  {n:integer):integer};
       begin
         CFACI:=N;
       end;

    {------------------------ draw a text string ---------------------------}

    FUNCTION STX   {(X,Y,COL:INTEGER; S:STRING):INTEGER} ;
       VAR I:INTEGER;
       BEGIN
         SETCOLOR(COL); OUTTEXTXY(X+FDX,Y+FDY,S);
       END;

    {--------------------------- draw a pixel ------------------------------}

    FUNCTION SMK   {(X,Y:INTEGER):INTEGER}  ;
       BEGIN
         PUTPIXEL(X+FDX,Y+FDY,3);
       END;

    {-------------------------- draw a fat pixel ---------------------------}

    FUNCTION SMK2   {(X,Y:INTEGER):INTEGER}  ;
       BEGIN
         MOVETO(X+FDX-1,Y+FDY+1);
         SETCOLOR(WHITE);
         LINETO(X+FDX+1,Y+FDY+1);
         LINETO(X+FDX+1,Y+FDY-1);
         LINETO(X+FDX-1,Y+FDY-1);
         LINETO(X+FDX-1,Y+FDY+1);
       END;

    {------------------------ initialize graphic mode ----------------------}

    PROCEDURE OUVREFENETRES;
       VAR I:INTEGER; S,S1:STRING;
           ecran,mode,code:INTEGER;
       BEGIN
         {turbo pascal library initialisation}
         ecran:=VGA; mode:=VGAHI;
         INITGRAPH(ecran,mode,'.');
         code:=GRAPHRESULT;
         IF code<>0 THEN
         BEGIN
           WRITELN('Graphical mode initialisation failed :',code);
           WRITELN('Is your screen VGA compatible ? Is EGAVGA.BGI in the current directory ?');
           HALT;
         END;

         {draw borders of windows}
         SETCOLOR(WHITE);
         RECTANGLE(0,0,499,331);
         RECTANGLE(0,333,499,479);
         RECTANGLE(501,0,639,479);

         {draw signals labels}
         FOR I:=1 TO NBMENTREES DO
           BEGIN
             S:=' '; S[1]:=CHR(ORD('0')+I);
             S1:=CONCAT('O',S); S:=CONCAT('I',S);
             SETCOLOR(LIGHTCYAN);
             MOVETO(6,340+8*(NBMSORTIES-NBMENTREES-1+I)); OUTTEXT(S);
             SETCOLOR(LIGHTGREEN);
             MOVETO(6,348+8*(NBMSORTIES+I-1)); OUTTEXT(S1);
           END;
       END;

    {------------------------ graphical mode closing -----------------------}

    PROCEDURE FERMEFENETRES;
       BEGIN
         RESTORECRTMODE;
       END;

 {--------------------------- Unit initialisation --------------------------}

 BEGIN
   CR:=SACWK(FSCHEMA);
   CLIGNE:=1;CFACI:=1;
 END.

{==========================================================================}
{                              END OF FILE                                 }
{==========================================================================}
