{==========================================================================}
{ Program            : MOSSIM                                              }
{ Author             : Robert Lacoste, Chaville, France                    }
{ Version            : 1.3                                                 }
{ Release date       : 27/07/1996                                          }
{==========================================================================}

{--------------------------------------------------------------------------}
{                              UNIT PROCGEN                                }
{                                                                          }
{ Description : Generic data manipulation procedures                       }
{                                                                          }
{--------------------------------------------------------------------------}

 UNIT PROCGEN;

   INTERFACE                    {public declarations}
     USES GRAPH,
          CRT,
          DOS,
          DATA,
          GDS;

     TYPE SETCHAR = SET OF CHAR;

     {max of two integers}
     FUNCTION  MAX(A,B:INTEGER):INTEGER;

     {int to string conversion (4 digits)}
     PROCEDURE CHAINEINT(X:INTEGER;VAR S:STRING);

     {int to string conversion (7 digits)}
     PROCEDURE CHAINEINT7(X:LONGINT;VAR S:STRING);

     {int to string conversion (1 digit)}
     PROCEDURE CHDIG(X:INTEGER;VAR S:STRING);

     {title zone blanking}
     PROCEDURE EFFACETITRE;

     {information zone blanking}
     PROCEDURE EFFACEINFO;

     {display a message in the title zone}
     PROCEDURE TITRE(VAR S:STRING);

     {display a message in the information zone}
     PROCEDURE INFO(VAR S:STRING;COL:INTEGER);

     {display an error number}
     PROCEDURE ERREUR(N:INTEGER);

     {ask for a confirmation}
     FUNCTION  CONFIRME:BOOLEAN;

     {grid row to screen coordinate}
     FUNCTION  XGRILLE(X:INTEGER):INTEGER;
     FUNCTION  YGRILLE(Y:INTEGER):INTEGER;

     {check if mouse driver available}
     function sourispresente:boolean;

     {wait for a mouse click somewhere}
     FUNCTION LITSOURIS(VAR X,Y:INTEGER):integer;

     {wait for the keyboard input of a N digit number}
     FUNCTION LITNUM(nbdig:integer):integer;

     {wait for the keyboard input of a string}
     PROCEDURE  LITTXT(VAR S:STRING; PROMPT:string; NBDIG:INTEGER;
                       AUTHOCHAR:SETCHAR);

     {put all signals to "invalid" state}
     PROCEDURE ETATSNULS;

     {put all outputs to "invalid" state}
     PROCEDURE SORTIESNULLES;

     {frees a node}
     PROCEDURE DISPOSENOEUD(N:INTEGER);

     {check if memory available}
     FUNCTION  ILYADELAPLACE(NBTR,NBND:INTEGER):BOOLEAN;

     {create a new node}
     FUNCTION  FAITNOEUD(CL:TCLASSE):ZONENOEUD;

     {check if screen coordinate free}
     FUNCTION  TESTESILIBRE(X,Y,MX,PX,MY,PY:INTEGER):BOOLEAN;

     {delare a screen aera as used}
     PROCEDURE METTABLE(X,Y,MX,PX,MY,PY:INTEGER; EL:TYPELEMENT; NUM:INTEGER);

     {declare that a modification was done}
     PROCEDURE MODIFEFFECTUEE;

     {copyright and statistics display}
     PROCEDURE STATISTICS;

     {display of signals states}
     PROCEDURE AFFICHESIGNAUX;

   IMPLEMENTATION              {local declarations}

 {------------------------- maximum of two integers -----------------------}

 function max  {(a,b:integer):integer};
   begin
     if a<b then max:=b else max:=a;
   end;

 {----------------------- int to string conversion (4 digits)--------------}

 procedure chaineint {(x:integer;var s:string)};
   begin
     S:='    ';
     s[1]:=chr(ord('0')+((x mod 10000) div 1000));
     S[2]:=chr(ORD('0')+((x mod 1000) div 100));
     s[3]:=chr(ORD('0')+((x mod 100) div 10));
     s[4]:=chr(ORD('0')+(x mod 10));
 end;

 {----------------------- int to string conversion (5 digits)--------------}

 procedure chaineint7 {(x:longint;var s:string)};
   begin
     S:='       ';
     s[1]:=chr(ord('0')+((x mod 10000000) div 1000000));
     s[2]:=chr(ord('0')+((x mod 1000000) div 100000));
     s[3]:=chr(ord('0')+((x mod 100000) div 10000));
     s[4]:=chr(ord('0')+((x mod 10000) div 1000));
     S[5]:=chr(ORD('0')+((x mod 1000) div 100));
     s[6]:=chr(ORD('0')+((x mod 100) div 10));
     s[7]:=chr(ORD('0')+(x mod 10));
 end;

 {----------------------- int to string conversion (1 digit) --------------}

 procedure chdig {(x:integer; var s:string)};
   begin
     S:=' ';
     S[1]:=chr(ORD('0')+(x mod 10));
   end;

 {----------------------- title zone blanking -----------------------------}

 procedure effacetitre;
   begin
     cr:=sacwk(ftexte);cr:=ssfaci(0);
     cr:=sgr(2,392,135,392+8) ;
   end;

 {----------------------- info zone blanking ------------------------------}

 procedure effaceinfo;
   begin
     cr:=sacwk(ftexte);cr:=ssfaci(0);
     cr:=sgr(2,406,135,406+8) ;
   end;

 {----------------------- Diplay a title ----------------------------------}

 procedure titre {(s:string)};
   begin
     cr:=sacwk(ftexte);
     effacetitre;
     CR:=stx(8,392,WHITE,s);
   end;

 {----------------------- Diplay an info ----------------------------------}

 procedure info {(s:string,col:integer)};
   begin
     cr:=sacwk(ftexte);
     effaceinfo;
     CR:=stx(8,406,col,s);
   end;

 {----------------------- Display an error number -------------------------}

 { error 1  : screen aera not free }
 { error 2  : output number already used}
 { error 3  : input number already used}
 { error 5  : file non existent}
 { error 10 : too much transistors }
 { error 12 : too much nodes }
 { error 20 : not a connected node}
 { error 22 : too many segments}
 { error 23 : circular connection}
 { error 31 : impossible connection (incompatible node class)}
 { error 32 : impossible connection (2 pullups)}
 { error 33 : impossible connection (2 outputs)}
 { error 34 : deletion impossible, no object there}
 { error 40 : bad file terminator}
 { error 44 : too many free texts}
 { error 60 : no stability reached when calculating nodes states}
 { error 61 : forced states conflict (conception error)}
 { error 94 : software error (error in node/transistor allocations)}
 { error 95 : software error (error in node number allocation)}
 { error 98 : software error (error in transistor allocation)}
 { error 99 : software error (error in node allocation)}

 PROCEDURE ERREUR  {(N:INTEGER)};
   CONST NBERR=17;
   TYPE errlib=record
                 num:0..99;
                 lib:string[15];
               end;
   CONST tberr:array[1..NBERR] of errlib =
         (
          (num:01;lib:'ALREADY USED'),
          (num:02;lib:'OUTPUT NUM USED'),
          (num:03;lib:'INPUT NUM USED'),
          (num:05;lib:'FILE NOEXISTANT'),
          (num:10;lib:'TOO MUCH TRANS.'),
          (num:12;lib:'TOO MUCH NODES'),
          (num:20;lib:'NOT A NODE'),
          (num:22;lib:'TOO MANY SEGS'),
          (num:23;lib:'CIRCULAR CONN.'),
          (num:31;lib:'INCOMPAT.CLASS'),
          (num:32;lib:'TWO PULLUPS'),
          (num:33;lib:'TWO OUTPUTS'),
          (num:34;lib:'NOTHING THERE'),
          (num:40;lib:'BAD FORMAT'),
          (num:44;lib:'TOO MANY TEXTS'),
          (num:60;lib:'NO STABILITY...'),
          (num:61;lib:'STATES CONFLICT')
         );
   VAR I,j,k:INTEGER;
       SS,s:string;
   BEGIN
     CR:=SACWK(FTEXTE);
     chdig((n DIV 10) mod 10,s); CHDIG(N MOD 10,SS); S:=CONCAT(S,SS);
     S:=CONCAT('ERROR ',s);

     {find the error string}
     ss:=s;
     for i:=1 to NBERR do
       if tberr[i].num=n then ss:=tberr[i].lib;

     sound(100);
     FOR i:=1 to 2 DO
         begin
               {flash the error number}
               EFFACEINFO;
               delay(400);
               INFO(S,LIGHTRED);
               delay(1200);
               nosound;
         end;

     FOR i:=1 to 5 DO
         begin
               {flash 5 times the error string}
               EFFACEINFO;
               delay(400);
               INFO(SS,LIGHTRED);
               delay(1200);
         end;
   END;

 {----------------------- Ask for a confirmation --------------------------}

 function confirme {:boolean};
   var w:boolean;
       S:STRING;
       ch:char;
   begin
     cr:=sacwk(ftexte);
     S:='Ok(o/n)?.'; INFO(S,WHITE);
     REPEAT ch:=ReadKey until (ch in ['o','O','n','N']);
     confirme:=(ch='o') or (ch='O');
   end;

 {------------------------ Grid to screen coordinates change --------------}

 function xgrille {(x:integer):integer};
   begin
     xgrille:=x*ECART;
   end;

 function ygrille{(y:integer):integer};
   begin
     ygrille:=y*ECART+1;
   end;

 {---------------- Check if mouse driver available ------------------------}

 function sourispresente {:boolean};
   CONST MOUSE=51;              {mouse interrupt number}
   VAR r:registers;

   begin
     {call mouse status function}
     r.ax:=0; intr(MOUSE,r);
     sourispresente := (r.ax = 65535);
   end;

 {---------------- Wait for a mouse click in network window ---------------}
 { Use Microsoft compatible mouse driver                                   }
 { x,y : screen coordinates where mouse clicked                            }
 { return mouse button number                                              }

 function litsouris {(var x,y:integer):integer};

   CONST MOUSE=51;              {mouse interrupt number}
   VAR CH:CHAR;
       r:registers;
       i:integer;

   {display of a cross in reserve color}
   PROCEDURE CROIXINV;
     BEGIN
       SETWRITEMODE(1); SETCOLOR(WHITE);
       MOVETO(XGRILLE(X)-5,YGRILLE(Y));
       LINETO(XGRILLE(X)+5,YGRILLE(Y));
       MOVETO(XGRILLE(X),YGRILLE(Y)+5);
       LINETO(XGRILLE(X),YGRILLE(Y)-5);
       SETWRITEMODE(0);
     END;

   begin

     {wait for mouse release}
     REPEAT
       r.ax:=3; intr(MOUSE,r);
     UNTIL r.bx=0;
     cr:=sacwk(fschema);
     X:=-1;

     {Define active mouse aera}
     r.ax:=7; r.cx:=1; r.dx:=2*DIMMX-1; intr(MOUSE,r);
     r.ax:=8; r.cx:=1; r.dx:=2*DIMMY-1; intr(MOUSE,r);

     {wait for a mouse click (any button)}
     REPEAT
       if (x>=0) THEN CROIXINV;
       r.ax:=3; intr(MOUSE,r);
       delay(100);
       if (x>0) THEN CROIXINV;
       x:=r.cx DIV 2+1; y:=r.dx DIV 2+1;
     UNTIL (r.bx<>0);
     litsouris:=r.bx;
   end;

 {----------- wait for the keyboard input of a nbdig digits number --------}

 function litnum{(nbdig:integer):integer};
   VAR S:STRING;
       I,C:INTEGER;
   begin
     {read input}
     LITTXT(s,'Num',nbdig,['0'..'9']);

     {return value}
     VAL(S,I,C);
     LITNUM:=I;
   end;

 {------------- wait for the keyboard input of a string -------------------}

 PROCEDURE  LITTXT{(VAR S:STRING; PROMPT:STRING; NBDIG:INTEGER);
                    AUTOCHAR:SET OF CHAR};
   VAR SS:STRING;
       A,I,Y,xdepart:INTEGER;
       C:CHAR;
   begin
     {display prompt}
     Y:=406;
     SS:=CONCAT(prompt,':');
     CR:=SACWK(FTEXTE); EFFACEINFO; CR:=STX(8,y,WHITE,SS);
     XDEPART:=8+LENGTH(SS)*8;
     SS:=COPY('......................',1,NBDIG);
     CR:=STX(xdepart,y,WHITE,SS);
     S:='';
     cr:=ssfaci(0);
     {wait for each character}
     I:=1;
     repeat
       BEGIN
         REPEAT c:=READKEY
         UNTIL ((C IN AUTHOCHAR)
               OR (C = chr(13)) OR (C=chr(8)));

         {if backspace, try to delete the last character entered}
         if (c = chr(8)) then
           begin
             if (i>1) then
               begin
                 s:=copy(s,1,length(s)-1);
                 CR:=SGR(xdepart+8*(I-2),y+8,xdepart+8*(i-2)+7,y);
                 CR:=STX(xdepart+8*(I-2),y,WHITE,'.');
                 i:=i-1;
               end;
           end

         {if not CR, add it to the string}
         else if (c <> chr(13)) then
           begin
             ss[1]:=c; ss:=copy(ss,1,1); s:=concat(s,ss);

             CR:=SGR(xdepart+8*(I-1),y+8,xdepart+8*(i-1)+7,y+4);
             CR:=STX(xdepart+8*(I-1),y,WHITE,SS);

             i:=i+1;
           END;
       end
     until (i>NBDIG) or (C=chr(13));
   end;

 {------------------------- put all nodes to "invalid" state --------------}

 procedure etatsnuls;
   var i:integer;
   begin
     for i:=1 to NBMNOEUDS do if noeuds[i]<>nil then noeuds[i]^.etat:=nondef;
   end;


 {------------------------- put all outputs to "invalid" state ------------}

 procedure sortiesnulles;
   var i,j:integer;
   begin
     for i:=1 to NBMSORTIES do
       for j:=1 to LONGSIGN do valsorties[i,j]:=nondef;
   end;

 {-------------------------- frees a node ---------------------------------}

 procedure disposenoeud {(n:integer)};
   begin
       noeuds[n]:=nil;
   end;

 {--------- check if memory available for new nodes or transistors --------}

 function ilyadelaplace {(nbtr,nbnd:integer):boolean};
   var ts:boolean;
   begin
     ts:=false;
     if nbtr+nbtr>NBMTRANSIS	      then erreur(10)
       else if nbnoeud+nbnd>NBMNOEUDS then erreur(12)
       else ts:=true;
     ilyadelaplace:=ts;
   end;

 {------------------------- create a new node -----------------------------}
 { return the new node number                                              }

 function faitnoeud {(cl:tclasse):zonenoeud};
   var pt:ptnoeud;
       index,i:integer;
   begin
     {increment nodes count}
     nbnoeud:=succ(nbnoeud);

     { get first free node record}
     index:=0; i:=1;
     if nbnoeud<=NBMNOEUDS then repeat
 				 if noeuds[i]=nil then index:=i;
 				 i:=succ(i);
     until ((i>NBMNOEUDS) or (index<>0));

     if index=0 then
       begin
         erreur(99);
         halt;
       end
     else
       {create and initialise a new node}
       begin
        new(pt);
        noeuds[index]:=pt;
        pt^.classe:=cl;
        pt^.debutconnections:=nil;
        pt^.numsortie:=0;
        PT^.XSORT:=0; PT^.YSORT:=0; pt^.etat:=nondef;
        pt^.debutsegments:=nil;
        pt^.xpull:=0; pt^.ypull:=0;
      end;
      faitnoeud:=index;
   end;

 {-------------------- check if screen aera free --------------------------}
 { zone checked : from x-mx to x+px and y-my to y+py                       }

 function testesilibre{(x,y,mx,px,my,py:integer):boolean};
   var i,j:integer;
       t:boolean;
   begin
     t:=true;
     for i:=x-mx to x+px do
       for j:=y-my to y+py do t:=t and (table[i,j].element=libre);
     if not(t) then erreur(1);
     testesilibre:=t;
   end;

 {-------------------- define a screen aera as used ----------------------}
 { from x-mx to x+px and y-my to y+py, used for element "el", number "num }

 procedure mettable{(x,y,mx,px,my,py:integer; el:typelement; num:integer)};
   var i,j:integer;
   begin
     for i:=x-mx to x+px do
       for j:=y-my to y+py do with table[i,j] do
   	     begin
 	       element:=el;
 	       case el of
 	         eltrans: numtr:=num;
 	         elpull,elvcc,elvss,elsort,elentr: numnoeud:=num;
             eltxt:numtxt:=num;
 	       end;
 	     end;
   end;

 {------------------ define that some modification was done --------------}

 PROCEDURE MODIFEFFECTUEE;
   BEGIN
     {change flag value}
     MODIFIE:=TRUE;

     {raz signals and states}
     SORTIESNULLES; ETATSNULS;

     {if signals displayed, erase them}
     if (sortiescalculees) then
     begin
       sortiescalculees:=false;
       AFFICHESIGNAUX;
     end;
   END;

 {--------------------- Copyright ans statistics writing ------------------}

 PROCEDURE STATISTICS;
   VAR s,ss:string;
       y:integer; l:longint;
   BEGIN
     cr:=sacwk(ftexte);

     {clear statistics zone}
     cr:=ssfaci(0);
     cr:=sgr(2,328,135,328+4*12+8);

     y:=328;
     cr:=stx(8,y,YELLOW,'Statistics :');
     y:=y+12;

     { total transistors number }
     chaineint(nbtr,s); S:=COPY(S,2,3);
     s:=concat('trans : ',s);
     chaineint(NBMTRANSIS,ss); SS:=COPY(SS,2,3);
     s:=concat(s,'/');s:=concat(s,ss);
     cr:=stx(8,y,YELLOW,s);
     y:=y+12;

     { total nodes number }
     chaineint(nbnoeud,s); S:=COPY(S,2,3);
     s:=concat('nodes : ',s);
     chaineint(NBMNOEUDS,ss); SS:=COPY(SS,2,3);
     s:=concat(s,'/');s:=concat(s,ss);
     cr:=stx(8,y,YELLOW,s);
     y:=y+12;

     {memory available}
     l:=memavail;
     chaineint7(l,s);
     s:=concat('memory: ',s);
     cr:=stx(8,y,YELLOW,s);
     y:=y+12;

     { copyright }
     y:=438;
     cr:=stx(8,y,LIGHTCYAN,'MOSSIM V1.3 (c)');
     y:=y+12;
     cr:=stx(8,y,LIGHTCYAN,'Robert LACOSTE');
     y:=y+12;
     cr:=stx(8,y,LIGHTCYAN,'France - 1996');
   END;

   {------------------------- display of the signals levels ---------------}

   PROCEDURE AFFICHESIGNAUX;
   var i,j,precc,xj,yj,v:integer;
       precc2:tristate;

   begin
     {clear signals window and draw separating lines}
     cr:=sacwk(fsignaux); cr:=sclrwk;
     cr:=ssplci(LIGHTGRAY);
     CR:=SLN(25,7,25,6+8*NBMENTREES);
     CR:=SLN(491,7,491,6+8*NBMENTREES);
     CR:=SLN(25,6,491,6);
     CR:=SLN(25,6+8*(NBMENTREES+1),491,6+8*(NBMENTREES+1));
     CR:=SLN(25,6+8*NBMENTREES+1,491,6+8*NBMENTREES+1);
     CR:=SLN(25,6+8*(NBMENTREES+NBMSORTIES+1),
             491,6+8*(NBMENTREES+NBMSORTIES+1));
     CR:=SLN(491,6+8*(NBMENTREES+1),491,142);
     CR:=SLN(25,6+8*(NBMENTREES+1),25,142);

     cr:=ssplci(LIGHTCYAN);
     {for each input}
     for i:=1 to NBMENTREES do
 	  begin
        {calculate vertical position}
 	    v:=6+i*ECARTVSIGN-2;

        {draw each time slot value}
 	    precc:=valentrees[i,1];
 	    for j:=1 to LONGSIGN do
 	      begin
 		    xj:=26+(j-1)*ECARTHSIGN;
 	   	    if valentrees[i,j]<>precc then cr:=sln(xj,v,xj,v-ECARTVSIGN+4);
 		    yj:=v-valentrees[i,j]*(ECARTVSIGN-4);
 		    cr:=sln(xj,yj,xj+ECARTHSIGN,yj);
 		    precc:=valentrees[i,j];
 	      end;
 	  end;

     {for each output}
     cr:=ssplci(LIGHTGREEN);
     cr:=ssfaci(GREEN);
     for i:=1 to NBMSORTIES do
 	  begin
        {calculate vertical position}
 	    v:=14+(i+NBMENTREES)*ECARTVSIGN-2;

        {draw each time slot value}
 	    precc2:=valsorties[i,1];
 	    for j:=1 to LONGSIGN do
 	      begin
 		    xj:=26+(j-1)*ECARTHSIGN;
 		    yj:=v-ECARTVSIGN+4;
 	        if valsorties[i,j]<>precc2 then cr:=sln(xj,v,xj,v-ECARTVSIGN+4);
 		    case valsorties[i,j] of
 		      zero: cr:=sln(xj,v,xj+ECARTHSIGN,v);
 		      un:   cr:=sln(xj,yj,xj+ECARTHSIGN,yj);
 		      nondef: cr:=sgr(xj,v,xj+ECARTHSIGN,yj);
 		    end;
 		    precc2:=valsorties[i,j];
 	      end;
 	  end;
   end;

{------------------------- Unit initialisation ----------------------------}

BEGIN
END.

{==========================================================================}
{                              END OF FILE                                 }
{==========================================================================}

